#!/usr/bin/env python
# -*- coding: utf-8 -*-
from builtins import object

import psycopg2
from db_samvaad import db_credentials as creds
from db_samvaad.get_secret import get_dbsecret
from psycopg2._psycopg import DatabaseError


class DatabaseHandle():

    def db_con(self):
        '''
        This function makes the connection with the db
        '''
        db_password = str(eval(get_dbsecret()).get("password"))
        db = psycopg2.connect(database=creds.DATABASE_NAME,
                              user=creds.DATABASE_USER,
                              password=db_password,
                              host=creds.DATABASE_HOST,
                              port=creds.DATABASE_PORT)
        return db

    __db = None

    def __init__(self):
        if DatabaseHandle.__db is None:
            DatabaseHandle.__db = self.db_con()

    @staticmethod
    def close():
        '''
        This method is used to close the database connection
        '''
        DatabaseHandle.__db.close()

    @property
    def db(self):
        return DatabaseHandle.__db


db = DatabaseHandle().db


class DBMethod():
    @staticmethod
    def get_value_from_iter_id(iter_id, column_name):
        '''
            This function is used to get the value of a column from iteration table

        PARAMETERS
        -------------
        iter_id
            iteration id whose value needs to be fetched

        column_name
            column  name whose value needs to be fetched
        '''

        cur = db.cursor()
        query = 'select {} from iteration where id={}'
        argument = (column_name, iter_id)
        try:
            cur.execute(query.format(*argument))
            column_value = cur.fetchone()[0]

            return column_value
        except DatabaseError as err:
            raise DatabaseError(str(err))

    @staticmethod
    def update_state(iter_id, state):
        '''
            To update the state of an iteration  with iter_id

        PARAMETERS
        ------------
        iter_id
            Iteration id whose state it to updated

        state
            New state of the iteration

        '''
        cur = db.cursor()
        query = "update iteration set state=%s where id=%s"
        arguments = (state, iter_id)
        row_affected = 0
        try:
            cur.execute(query, arguments)
            row_affected = cur.rowcount
            db.commit()
        except DatabaseError as err:
            db.rollback()
            raise DatabaseError(str(err))
        return row_affected

    @staticmethod
    def update_heartbeat(iter_id, percentage):
        '''
            To update the heartbeat of an iteration  with iter_id

        PARAMETERS
        ------------
        iter_id
            Iteration id whose state it to updated

        state
            New heartbeat of the iteration

        '''

        cur = db.cursor()
        query = "update iteration set heartbeat=%s where id=%s"
        arguments = (percentage, iter_id)
        row_affected = 0
        try:
            cur.execute(query, arguments)
            row_affected = cur.rowcount
            db.commit()
        except DatabaseError as err:
            db.rollback()
            raise DatabaseError(str(err))
        return row_affected

    @staticmethod
    def get_task_name_from_algo(algo_name, column_name):
        '''
            This function is to get the ecs_Task name from algorithm name

        PARAMETERS
        -----------
        algo_name
            name of the algorithm

        column_name
            column whose value is to be fetched
        '''
        cur = db.cursor()
        query = "select ecs_task from algorithm where name=%s"
        arguments = (algo_name,)
        try:
            cur.execute(query, arguments)
            task_id = cur.fetchone()[0]
            task_name = self.get_task_name(task_id, column_name)
            return task_name
        except DatabaseError as err:
            raise DatabaseError(str(err))

    @staticmethod
    def get_task_name(task_id, column_name):
        '''
            To get the task name of the task with id task_id

        PARAMETERS
        ------------
        task_id
            Id of the task whose name is to be fetched

        column_name
            column whose value is to be fetched
        '''

        cur = db.cursor()
        query = "select {} from ecs_task where id={}"
        arguments = (column_name, task_id)
        try:
            cur.execute(query.format(*arguments))
            row = cur.fetchone()[0]
            return row
        except DatabaseError as err:
            raise DatabaseError(str(err))

    @staticmethod
    def get_value(table_name, column_name, pr_key):
        '''
        This method returns the value of a particular column with given name(column_name)
        from a given table(table_name) with the primary key(pr_key)

        PARAMETERS
        ----------
        table_name : str
            table name from which the data is to be fetched
        column_name : str
            column name whose value is to be fetched
        pr_key : int
            primary key of the row in the table

        RETURNS
        --------
        row: tuple
            tuple which contains the value of a particular row with the primary
            key pr_key
        '''
        cur = db.cursor()
        query = 'select {1} from {0} where id={2} '
        arguments = (table_name, column_name, pr_key)
        try:
            cur.execute(query.format(*arguments))
            row = cur.fetchone()[0]
            return row
        except DatabaseError as err:
            raise DatabaseError(str(err))

    def update_value(self, table_name, column_name, new_value, pr_key):
        '''
        This method modifies the value of a particular column with given name(column_name)
        from a given table(table_name) with the primary key(pr_key)

        PARAMETERS
        ----------
        table_name : str
            table name from which the data is to be fetched
        column_name : str
            column name whose value is to be fetched
        new_value : str
            new value  of the column
        pr_key : int
            primary key of the row in the table

        RETURNS
        --------
        new_value: str |int |json
            updated value of the column
        '''
        cur = db.cursor()
        query = "update {} set {}='{}' where id={}"
        arguments = (table_name, column_name, new_value, pr_key)
        try:
            cur.execute(query.format(*arguments))
            new_value = None
            db.commit()
            if cur.rowcount:
                new_value = self.get_value(
                    table_name, column_name, pr_key=pr_key)
            return new_value
        except DatabaseError as err:
            db.rollback()
            raise DatabaseError(str(err))

    @staticmethod
    def execute_query(query, arguments):
        '''
            This method runs the raw query on the database

        PARAMETERS
        -----------
        query : string
            query to be run on the database
        arguments : tuple
            tuple which contains the arguments for the query

        RETURNS
        --------
        rows: int| list
            if the query is DML then it return the integer i.e number of updated rows
            if the query if DQL then it returns the query result
        '''
        cur = db.cursor()
        try:
            cur.execute(query.format(*arguments))
            if "select" in query:
                rows = cur.fetchall()
            else:
                rows = cur.rowcount
                db.commit()
            return rows
        except DatabaseError as err:
            raise DatabaseError(str(err))
